#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Multipath Sim
# Generated: Fri Feb  8 09:04:45 2013
##################################################

from PyQt4 import Qt
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio.eng_option import eng_option
from gnuradio.gr import firdes
from gnuradio.qtgui import qtgui
from optparse import OptionParser
from scipy import fftpack
import PyQt4.Qwt5 as Qwt
import math
import sip
import sys

class multipath_sim(gr.top_block, Qt.QWidget):

	def __init__(self):
		gr.top_block.__init__(self, "Multipath Sim")
		Qt.QWidget.__init__(self)
		self.setWindowTitle("Multipath Sim")
		self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
		self.top_scroll_layout = Qt.QVBoxLayout()
		self.setLayout(self.top_scroll_layout)
		self.top_scroll = Qt.QScrollArea()
		self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
		self.top_scroll_layout.addWidget(self.top_scroll)
		self.top_scroll.setWidgetResizable(True)
		self.top_widget = Qt.QWidget()
		self.top_scroll.setWidget(self.top_widget)
		self.top_layout = Qt.QVBoxLayout(self.top_widget)
		self.top_grid_layout = Qt.QGridLayout()
		self.top_layout.addLayout(self.top_grid_layout)


		##################################################
		# Variables
		##################################################
		self.tap3 = tap3 = 0.6
		self.tap2 = tap2 = 0.5
		self.tap1 = tap1 = 0.25
		self.sps = sps = 4
		self.nfilts = nfilts = 32
		self.taps = taps = fftpack.ifft([tap1+1j*tap1, tap2 - 1j*tap2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, -tap3 + 1j*tap3])
		self.samp_rate = samp_rate = 32000
		self.rrc_taps = rrc_taps = firdes.root_raised_cosine(nfilts, nfilts, 1.0/float(sps), 0.35, 11*sps*nfilts)
		self.arity = arity = 4

		##################################################
		# Blocks
		##################################################
		self._tap3_layout = Qt.QVBoxLayout()
		self._tap3_label = Qt.QLabel("tap3")
		self._tap3_slider = Qwt.QwtSlider(None, Qt.Qt.Vertical, Qwt.QwtSlider.LeftScale, Qwt.QwtSlider.BgSlot)
		self._tap3_slider.setRange(0, 1, 0.01)
		self._tap3_slider.setValue(self.tap3)
		self._tap3_slider.setMinimumHeight(200)
		self._tap3_slider.valueChanged.connect(self.set_tap3)
		self._tap3_label.setAlignment(Qt.Qt.AlignTop)
		self._tap3_layout.addWidget(self._tap3_slider)
		self._tap3_layout.addWidget(self._tap3_label)
		self.top_grid_layout.addLayout(self._tap3_layout, 0,13,1,1)
		self._tap2_layout = Qt.QVBoxLayout()
		self._tap2_label = Qt.QLabel("tap2")
		self._tap2_slider = Qwt.QwtSlider(None, Qt.Qt.Vertical, Qwt.QwtSlider.LeftScale, Qwt.QwtSlider.BgSlot)
		self._tap2_slider.setRange(0, 1, 0.01)
		self._tap2_slider.setValue(self.tap2)
		self._tap2_slider.setMinimumHeight(200)
		self._tap2_slider.valueChanged.connect(self.set_tap2)
		self._tap2_label.setAlignment(Qt.Qt.AlignTop)
		self._tap2_layout.addWidget(self._tap2_slider)
		self._tap2_layout.addWidget(self._tap2_label)
		self.top_grid_layout.addLayout(self._tap2_layout, 0,12,1,1)
		self._tap1_layout = Qt.QVBoxLayout()
		self._tap1_label = Qt.QLabel("tap1")
		self._tap1_slider = Qwt.QwtSlider(None, Qt.Qt.Vertical, Qwt.QwtSlider.LeftScale, Qwt.QwtSlider.BgSlot)
		self._tap1_slider.setRange(0, 1, 0.01)
		self._tap1_slider.setValue(self.tap1)
		self._tap1_slider.setMinimumHeight(200)
		self._tap1_slider.valueChanged.connect(self.set_tap1)
		self._tap1_label.setAlignment(Qt.Qt.AlignTop)
		self._tap1_layout.addWidget(self._tap1_slider)
		self._tap1_layout.addWidget(self._tap1_label)
		self.top_grid_layout.addLayout(self._tap1_layout, 0,11,1,1)
		self.qtgui_sink_x_0 = qtgui.sink_c(
			1024, #fftsize
			firdes.WIN_BLACKMAN_hARRIS, #wintype
			0, #fc
			samp_rate, #bw
			"QT GUI Plot", #name
			True, #plotfreq
			True, #plotwaterfall
			True, #plottime
			True, #plotconst
		)
		self._qtgui_sink_x_0_win = sip.wrapinstance(self.qtgui_sink_x_0.pyqwidget(), Qt.QWidget)
		self.top_grid_layout.addWidget(self._qtgui_sink_x_0_win, 0,0,1,10)
		self.gr_throttle_0 = gr.throttle(gr.sizeof_gr_complex*1, samp_rate)
		self.gr_noise_source_x_0 = gr.noise_source_c(gr.GR_GAUSSIAN, 1, 0)
		self.gr_channel_model_0 = gr.channel_model(
			noise_voltage=0.01,
			frequency_offset=0,
			epsilon=1.0,
			taps=(taps),
			noise_seed=0,
		)

		##################################################
		# Connections
		##################################################
		self.connect((self.gr_noise_source_x_0, 0), (self.gr_throttle_0, 0))
		self.connect((self.gr_throttle_0, 0), (self.gr_channel_model_0, 0))
		self.connect((self.gr_channel_model_0, 0), (self.qtgui_sink_x_0, 0))

	def get_tap3(self):
		return self.tap3

	def set_tap3(self, tap3):
		self.tap3 = tap3
		self._tap3_slider.setValue(self.tap3)
		self.set_taps(fftpack.ifft([self.tap1+1j*self.tap1, self.tap2 - 1j*self.tap2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, -self.tap3 + 1j*self.tap3]))

	def get_tap2(self):
		return self.tap2

	def set_tap2(self, tap2):
		self.tap2 = tap2
		self._tap2_slider.setValue(self.tap2)
		self.set_taps(fftpack.ifft([self.tap1+1j*self.tap1, self.tap2 - 1j*self.tap2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, -self.tap3 + 1j*self.tap3]))

	def get_tap1(self):
		return self.tap1

	def set_tap1(self, tap1):
		self.tap1 = tap1
		self._tap1_slider.setValue(self.tap1)
		self.set_taps(fftpack.ifft([self.tap1+1j*self.tap1, self.tap2 - 1j*self.tap2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, -self.tap3 + 1j*self.tap3]))

	def get_sps(self):
		return self.sps

	def set_sps(self, sps):
		self.sps = sps
		self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.sps), 0.35, 11*self.sps*self.nfilts))

	def get_nfilts(self):
		return self.nfilts

	def set_nfilts(self, nfilts):
		self.nfilts = nfilts
		self.set_rrc_taps(firdes.root_raised_cosine(self.nfilts, self.nfilts, 1.0/float(self.sps), 0.35, 11*self.sps*self.nfilts))

	def get_taps(self):
		return self.taps

	def set_taps(self, taps):
		self.taps = taps
		self.gr_channel_model_0.set_taps((self.taps))

	def get_samp_rate(self):
		return self.samp_rate

	def set_samp_rate(self, samp_rate):
		self.samp_rate = samp_rate
		self.qtgui_sink_x_0.set_frequency_range(0, self.samp_rate)

	def get_rrc_taps(self):
		return self.rrc_taps

	def set_rrc_taps(self, rrc_taps):
		self.rrc_taps = rrc_taps

	def get_arity(self):
		return self.arity

	def set_arity(self, arity):
		self.arity = arity

if __name__ == '__main__':
	parser = OptionParser(option_class=eng_option, usage="%prog: [options]")
	(options, args) = parser.parse_args()
	qapp = Qt.QApplication(sys.argv)
	tb = multipath_sim()
	tb.start()
	tb.show()
	qapp.exec_()
	tb.stop()

